/*
 *  $Id: issend.c,v 1.3 1994/06/07 21:22:16 gropp Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

/***********************************************************************
*                                                                      *
*   issend.c                                                           *
*   MPI for MS-Windows 3.1                                             *
*   current version: 0.99b          06/10/95                           *
*                                                                      *
*   Joerg Meyer                                                        *
*   University of Nebraska at Omaha (UNO)                              *
*   Department of Computer Science                                     *
*                                                                      *
*   This is an MPI implementation for MS-Windows 3.1                   *
*   It is based on the MPI implementation from Argonne National        *
*   Laboratory and Mississippi State University, version from          *
*   June 17, 1994. Note their COPYRIGHT.                               *
*   ( source code and user's guide available by anonymous FTP from     *
*     info.mcs.anl.gov in directory /pub/mpi )                         *
*   Anyone is free to copy and modify this code to suit his or her     *
*   own purposes as long as these notices are retained.                *
*                                                                      *
***********************************************************************/

#include <mpiimpl.h>
#include <mpisys.h>
#pragma hdrstop


#ifndef lint
static char vcid[] = "$Id: issend.c,v 1.3 1994/06/07 21:22:16 gropp Exp $";
#endif /* lint */

/*@
    MPI_Issend - Starts a nonblocking synchronous send

Input Parameters:
. buf - initial address of send buffer (choice) 
. count - number of elements in send buffer (integer) 
. datatype - datatype of each send buffer element (handle) 
. dest - rank of destination (integer) 
. tag - message tag (integer) 
. comm - communicator (handle) 

Output Parameter:
. request - communication request (handle) 

@*/
Int MPI_Issend( void far *buf, Int count, MPI_Datatype datatype, Int dest, 
					Int tag, MPI_Comm comm, MPI_Request far *request)
{
    Int err;
    if (dest != MPI_PROC_NULL)
    {
        /* We'll let MPI_Ssend_init find the errors */
		if ((err = MPI_Ssend_init( buf, count, datatype, dest, tag,
								   comm, request )) != 0)
		    return err;
		    
		return MPI_Start( request );
    }
    else 
		*request = 0;
		
    return MPI_SUCCESS;
}
